
CaseControlDipsim <-
  function(n=20,penetrance=c(0.00,0.7,1.0),sites=2010,var=200,freq=0.1,rho=0.1)
  {
    selectdip <- function(u,penetrance,n) {
      m <- rowSums(matrix(u,ncol=2,byrow=T))+1 
      pu <- numeric(length(m))
      pu <- penetrance[m]
      sel <-matrix(runif(length(m))<pu,nrow=2,ncol=length(m),byrow=T)
      seli <- as.vector(sel)
 #     cat("got ",sum(seli)/2," affected individuals\n")
      if (sum(seli)<n)
        stop("need larger penetrance or more sporadic cases to get ",n)
      list(case=which(seli)[1:n],control=which(!seli)[1:n])
    }
    fmn <- freq-0.02
    f <- c((1-fmn)^2,2*fmn*(1-fmn),fmn^2)%*%penetrance
    ss <- 2*round((n/2+4*sqrt(n/2))/f)
    cat("Choose a total sample size of ", ss/2,"individuals,\nwe expect to get"
        ,round(ss*f/2),"affected individuals\nsampling tree\n")
    a <- simARG(ss,sites,rho)
    ascert <- paste("panel(",as.integer(n/2),")",sep="")
    cat("mutating tree with ascertainment",ascert,"\n")
    b <- mutate(a,(var+1),ascert=ascert)
    
    f1 <- apply(b$d,2,mean)
    
 
    w1 <- which(abs(f1-freq)<0.02)
    if (length(w1)==1)
      stop("no sites with the correct frequency\n")
    wh <- w1[which(abs(w1-var/2) == min(abs(w1-var/2)))]
    if (length(wh)>1)
      wh <- sample(wh,1) ## possible to get two equally spaced
    pos <- b$position[wh]
   # cat("choose locus from {",paste(wh,collapse=","),"} with frequencies",f1[wh],"\n")
    u1 <- b$d[,wh]
    cc <- selectdip(u1,penetrance,n)
    a <- prune(a,c(cc$case,cc$control))
    ccdata <- list(ss=2*n,sites=var,graph=a,DiseasePos=pos,wh=wh,cc=cc
                   ,positions=b$position[-wh]
                   ,d=rbind(b$d[cc$case,-wh],b$d[cc$control,-wh])
                   ,locations=rep(1:2,c(n,n))
                   ,SNP=c(b$d[cc$case,wh],b$d[cc$control,wh])
                   )
    class(ccdata) <-c("CCDip","ARG","GenomicHaplotype")
    ccdata
  }


fixphylo <- function(phy,cc)
  {
    lab <- as.numeric(phy$tip.label)+1
    case <- lab %in% cc$case
    newlab <- lab
    newlab[case] <- "*"
    newlab[!case] <- " "
    phy$tip.label <- newlab
    phy
  }



#plot(tp50 <- fixphylo(tree(a1p,pos+50),cc1))

#tp100 <- fixphylo(tree(a1p,pos+100),cc1)
#tp200 <- fixphylo(tree(a1p,pos+200),cc1)
#tp300 <- fixphylo(tree(a1p,pos+300),cc1)
#1000 <- fixphylo(tree(a1p,1000),cc1)
#t1 <- fixphylo(tree(a1p,1),cc1)
#tm100 <- fixphylo(tree(a1p,pos-100),cc1)
#tm200 <- fixphylo(tree(a1p,pos-200),cc1)
#tm300 <- fixphylo(tree(a1p,pos-300),cc1)
